           .globl tower,output
;
; Loesung des Tuerme von Hanoi-Problems fuer beliebige Anfangssituation
; mit bis zu 63 Ringen.
; Aufruf mit JSR PC,TOWER, Parameteruebergabe ber den Stack:
; 1. Argument - Anzahl der Ringe
; 2. Argument - Zeiger auf Tabelle der Ringpositionen
;
tower:     mov  @2(sp),r0    ; Anzahl der Ringe vom Stack holen
           mov  4(sp),tab    ; Tabellenzeiger vom Stack holen
           mov  #'C,r1       ; Zielspindel
           jsr  pc,try
           rts  pc
;
; Rekursive Subroutine TRY:
; Bringe die Ringe 1 bis N auf die Spindel DEST.
; Parameteruebergabe: N in R0 (Word), DEST in R1 (Byte)
;
; Benutzt werden: R2 als Zeiger auf die Ring-Tabelle ab TAB
;                 R3 als Index I und als Zwischenspeicher
;                 R4 als neue Zielspindel NEW
;
try:       mov  tab,r2       ; setze R2 auf das N+1-te Element
           add  r0,r2        ;   der Ring-Tabelle
           tst  r0
desttest:  beq  endtry       ; beende TRY, wenn N=0
           cmpb -(r2),r1     ; liegt Ring N bereits auf Spindel DEST?
           bne  notok        ;   nein, also mu ab hier umsortiert werden
           dec  r0           ; wiederhole den Test mit N - 1
           br   desttest
;
; Hier zeigt R2 auf den groessten Ring zwischen 1 und dem urspruenglichen N,
; der noch nicht auf DEST liegt, N ist der zugehoerige Index.
;
notok:     mov  #'A+'B+'C,r4
           movb (r2),r3
           sub  r3,r4
;
; Die letzten beiden Zeilen ersetzen die nicht vorhandene Byte-Arithmetik.
; Haette der Befehl SUB ein Byte-Aequivalent koennte man einfach..
;
;          SUBB (R2),R4
;
; ...schreiben.
;
           sub  r1,r4
;
; Damit ist NEW also weder die momentane Position des N-ten Rings
; noch sein Ziel. Wenn die Ringe 1 bis N-1 alle auf Spindel NEW liegen,
; ist der Transport von Ring N nach Spindel DEST offensichtlich trivial.
;
           mov  r0,r3        ; I := N
testnew:   dec  r3           ; I := I - 1
           beq  move         ; I=0, trivialer Fall (s.o.)
           cmpb -(r2),r4     ; liegt Ring I auf Spindel NEW?
           beq  testnew      ;   ja, weitertesten
;
; Es folgt der nicht-triviale, rekursive Fall:
; Die Ringe 1 bis I muessen nun nach Spindel NEW gebracht werden, um die
; Bewegung des Rings N nach Spindel DEST zu ermoeglichen (*).
;
           mov  r0,-(sp)     ; N und DEST
           mov  r1,-(sp)     ;   auf den Stack bringen
           mov  r3,r0        ; N := I
           mov  r4,r1        ; DEST := NEW
           jsr  pc,try       ; rekursiver Aufruf von TRY wegen (*)
           mov  (sp)+,r1     ;   DEST und N
           mov  (sp)+,r0     ;     vom Stack zurueckholen
;
; Ab hier kann man in den trivialen Fall uebergehen:
; Bewege den Ring N von seinem jetzigen Platz nach Spindel DEST
;
move:      mov  tab,r2       ; setze R2 auf das N+1-te Element
           add  r0,r2        ;   der Ring-Tabelle
;
; Hier wird die Ausgabezeile fr die Routine OUTPUT vorbereitet.
; Angenommen wird N < 77 (oktal), d.h. hoechstens zwei Ziffern.
; Benutzt werden: R3 als Zeiger auf die Zeile
;                 R4 als Zwischenspeicher fr N
;
           mov  #line,r3     ; initialisiere R3
           movb -(r2),(r3)+  ; Buchstabe der Quellenspindel
           mov  r0,r4        ; R4 := N
           bic  #7,r4        ; teste, ob N > 7
           beq  digit        ;   nein, dann nur eine Ziffer ausgeben
           ash  #-3,r4       ; R4 enthaelt jetzt die linke Ziffer von N
           add  #'0,r4
           movb r4,(r3)+     ; 8er-Stelle der Ringnummer
digit:     mov  r0,r4        ; R4 := N
           bic  #70,r4       ; betrachte nur N mod 8
           add  #'0,r4
           movb r4,(r3)+     ; 1er-Stelle der Ringnummer
           movb r1,(r3)+     ; Buchstabe der Zielspindel
           mov  #' ,r4       ; vorsichtshalber noch ein Space
           movb r4,(r3)+     ;   hinterherschicken
           movb r1,(r2)      ; bringe die Tabelle auf den neuesten Stand
           mov  r0,-(sp)     ; Register vor
           mov  r1,-(sp)     ;   Aufruf von
           mov  r2,-(sp)     ;     OUTPUT sichern
           mov  #line,-(sp)  ; Argument fr OUTPUT
           jsr  pc,output    ; Ausgabe von LINE
           add  #2,sp        ; Stackkorrektur nach Aufruf von OUTPUT
           mov  (sp)+,r2     ; gesicherte
           mov  (sp)+,r1     ;   Register
           mov  (sp)+,r0     ;     zurueckholen
;
; Alle Ringe mit Nummern groesser oder gleich N liegen jetzt bereits
; richtig auf DEST. Die kleineren Ringe muessen evtl. noch
; bewegt werden:
;
           dec  r0           ; N := N - 1
           br   desttest
endtry:    rts  pc
;
; Datenbereich:
;
tab:       .blkw 1           ; Zeiger auf Tabellenanfang
line:      .ascii '                                        '
           .ascii '                                        '
                             ; Auszudruckende Zeile bei MOVE
           .end
